# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['magicdoc',
 'magicdoc.classes',
 'magicdoc.commands',
 'magicdoc.commands.tf_commands']

package_data = \
{'': ['*'], 'magicdoc': ['modules/*', 'templates/*']}

install_requires = \
['PyYaml>=5.3,<6.0',
 'click>=7.0,<8.0',
 'cloudmage-jinjautils>=1.0.2,<2.0.0',
 'colorama>=0.4.3,<0.5.0',
 'graphviz>=0.13.2,<0.14.0',
 'pyhcl>=0.4.0,<0.5.0',
 'python-terraform>=0.10.1,<0.11.0',
 'requests>=2.22.0,<3.0.0']

entry_points = \
{'console_scripts': ['magicdoc = magicdoc.main:cli']}

setup_kwargs = {
    'name': 'magicdoc',
    'version': '0.1.1',
    'description': 'MagicDoc is a Python command line tool/library that is designed to provide an extremely simple way to automatically create detailed documentation for certain project types.',
    'long_description': '# MagicDoc Automatated Project Documentation CLI Utility Package\n\n> __NOTE: This Project is currently IN Beta as of this latest push on 02/19/2020.__\n\n<br>\n\n## Description\n\nDocumentation is a tedious task that most of the time is either ignored or completed with minimal detail and effort, potentially making it difficult for others to consume and adequately use the project being developed. For example, when building IaC for cloud environments, it can become an aggravating burden to take the time to document each module or project required to create and test the actual infrastructure necessary for personal or business workloads. MagicDoc is a tool that aims to help resolve that very problem and help to lessen and eventually eliminate that very burden. The goal of this project is to be able to very quickly and easily produce detailed, quality documentation within minutes, and without the need to write any markdown manually. A command here, a few sentences there, and poof, just like magic, MagicDoc has saved hours of time and created beautiful documentation for your project!\n\n<br><br>\n\n## Python Version\n\nThis library is compatible with Python 3.6 and higher. It may work with earlier versions of Python3 but was not tested against anything earlier then 3.6. As Python 2.x is soon to be end of life, backward compatibility was not taken into consideration.\n\n<br><br>\n\n## Installation\n\nThis library has been published to [PyPi](https://pypi.org/project/magicdoc/) and can be installed via normal python package manager conventions such as [pip](https://pip.pypa.io/en/stable/) or [poetry](https://pypi.org/project/poetry/).\n\n<br>\n\n```python\npip3 install magicdoc\n```\n\n<br><br>\n\n## MagicDoc Commands\n\n```yaml\nmagicdoc:\n    Examples:\n        magicdoc --help\n    Arguments: None\n    Options: None\n```\n\n<br><br>\n\n## MagicDoc Provider Sub-Command Groups:\n\n```yaml\ntf:\n    Descripton: the tf subcommand sets the engine provider to terraform, loads the necessary terraform parsing libraries, and instantiates the necessary terraform objects to initialize terraform parsing capabilities.\n    Examples:\n        magicdoc tf\n        magicdoc tf --help\n    Arguments: None\n    Options:\n        verbose:\n            Description: Enables / Disables Verbose logging\n            Value: bool\n            Flag: --verbose, -v\n            Environment Variable: MAGICDOC_TF_VERBOSE\n            Required: No\n            Default: False\n        verbose_level:\n            Description: |\n                    All application processes use standard logging format to include DEBUG, INFO, WARNING, and ERROR. By passing the -l flag the user user can specify the level of verbosity that they wish to see. Passing -l INFO for example will exclude debug logs, yet still allow INFO, WARNING, and ERROR messages to display in the cli console.\n            Value: DEBUG | INFO | WARNING | ERROR\n            Flag: --logging_level, -l\n            Environment Variable: MAGICDOC_TF_VERBOSE_LEVEL\n            Required: No\n            Default: DEBUG\n        directory:\n            Description: |\n                    By default magicdoc will use the current directory path, however if an alternative directory is desired, then the -d flag can be used to specify the desired target directory path for the command execution.\n            Value: Must be valid directory path.\n            Flag: --dir, -d\n            Environment Variable: MAGICDOC_TF_DIRECTORY\n            Required: No\n            Default: Current Working Directory\n        exclude_dir:\n            Description: |\n                    When searching a directory location for terraform files to parse, in the event that a directory exists in the parent target directory, that is not a desired search directory, it can be excluded using the -e flag. If specified, then the file search operation will ignore the specified subdirectory if it exists.\n            Value: Must be valid directory path.\n            Flag: --exclude_dir, -e\n            Environment Variable: MAGICDOC_TF_EXCLUDE_DIR\n            Required: No\n            Default: None\n        config:\n            Description: |\n                    The -c flag allows the user to specify the location of the target project magicdoc configuration file. By default magicdoc will look for the presense of a `magicdoc.yaml` file. Users can name the config file by another filename when creating the config, however the file must be a yaml formatted file, and if an alternative filename is specified then this option must be issued in order for magicdoc to be able to effectively find the config file that will be used to generate the documentation.\n            Value: Must be valid file path.\n            Flag: --config, -c\n            Environment Variable: MAGICDOC_TF_CONFIG\n            Required: No\n            Default: magicdoc.yaml\n        no_recursion:\n            Description: |\n                    The -nr flag is used to tell magicdoc to not search any subdirectories in a target project path. If the -nr flag is set, then only the parent target directory will be searched for terraform files, which will also automatically exclude subdirectories from variables and outputs search.\n            Value: bool\n            Flag: --no_recursion, -nr\n            Environment Variable: MAGICDOC_TF_NO_RECURSION\n            Required: No\n            Default: True (Recursion enabled)\n    Available Sub-Commands:\n        - env\n        - show\n        - create\n```\n\n<br><br>\n\n## MagicDoc TF Show Env\n\nThe `magicdoc tf env` command will display information about the tf provider environment. It will display information such as if default template directory was found, what target directory path was specified, if a terraform object could be instantiated, meaning that magicdoc was able to find parsable terraform files in the target directory, and what flags where passed to the command. Its a simple check command for a quick look at how the tf provider environment is constructed when issued against a target project directory.\n\n<br>\n\n```yaml\nenv:\n    Examples:\n        magicdoc tf env\n    Arguments: None\n    Options: None\n```\n<br><br>\n\n## MagicDoc TF Show Commands\n\nThe `magicdoc tf show` command is a container for show subcommands. By itself this command will not return anything other then the help menu.\n\n```bash\nUsage: magicdoc tf show [OPTIONS] COMMAND [ARGS]...\n\nOptions:\n  --help  Show this message and exit.\n\nCommands:\n  files      Display the terraform target project file lists.\n  git        Display Terraform Project Git Config Data\n  graph      Display Terraform Project dot Graph Object\n  outputs    Display Terraform Project Outputs\n  release    Display Terraform Project Latest Release\n  repo       Display Terraform Project Git Repository Data\n  tree       Display Terraform Project Directory Tree\n  variables  Display Terraform Project Variables.\n```\n\n<br>\n\n### `magicdoc tf show files`\n\nThe `magicdoc tf show files` command will display all of the .tf and .tfvar files in the target project directory that magicdoc was able to find and will use to perform its search and parse operation for Terraform variables and outputs.\n\n<br>\n\n```yaml\nmagicdoc tf show files:\n  Examples:\n    magicdoc tf show files\n    magicdoc tf -d /path/to/module/sourcecode show files\n  Arguments: None\n  Options: None\n```\n\n<br>\n\n```bash\nMagicDoc Terraform Project File Summary:\n========================================\n\nMagicDoc [tf show files] Command Environment:\nGathering Terraform Project Files...\n\nTerraform file search target directory location: /Volumes/MacData/Work/CloudMage/Terraform/TF-AWS-CodeBuild-Module\n  -> 6 terraform file(s) found in target directory.\n  -> 1 tfvar file(s) found in target directory.\n\nTerraform .tf files:\n====================\noutputs.tf\nmain.tf\nvariables.tf\nexample/outputs.tf\nexample/main.tf\nexample/variables.tf\n\nTerraform .tfvar files:\n=======================\nexample/env.tfvars\n```\n\n<br><br>\n\n### `magicdoc tf show variables`\n\nThe `magicdoc tf show variables` command will display an output of all terraform variables that were found in the `variables.tf` file within the target project directory. Magicdoc will use the variables that it was able to parse from any found variable.tf files when constructing both the config file as well as the readme documentation.\n\n<br>\n\n```yaml\nmagicdoc show variables:\n  Examples:\n    magicdoc tf show variables\n    magicdoc tf -d /path/to/module/sourcecode show variables\n  Arguments: None\n  Options:\n      include_examples:\n          Description: Instructs the variables execution environment to include and parse files in any `example` or `examples` subdirectories located in the parent target project directory. By default directories named `example`, or `examples` are excluded from the file/variable/output search results, and are not parsed or included.\n          Value: bool\n          Flag: --include_examples, -1\n          Environment Variable: MAGICDOC_TF_SHOW_VARIABLES_INCLUDE_EXAMPLES\n          Required: No\n          Default: False\n```\n\n<br>\n\n```bash\n  Environment: Verbose Attribute Set:                False\n  Log: Verbose Attribute Set:                        False\n  Environment: Verbose Level Attribute Set:          debug\n\nMagicDoc Terraform Project Variable Summary:\n============================================\n  Include Example Directories:                       False\n\nMagicDoc [tf show variables] Command Environment:\nGathering Terraform Project Files...\n\nParsing variables.tf files for Terraform variables...\nScanning project directory for defined module variables...\nTerraform variable search target directory location: /Volumes/MacData/Work/CloudMage/Terraform/TF-AWS-CodeBuild-Module\n -> 8 required terraform project variables found in target project: /Volumes/MacData/Work/CloudMage/Terraform/TF-AWS-CodeBuild-Module\n -> 13 optional terraform project variables found in target project: /Volumes/MacData/Work/CloudMage/Terraform/TF-AWS-CodeBuild-Module\n\nTerraform Project Required Variables:\n=====================================\ncodebuild_project_name     = \'Required Value\'\ncodebuild_project_desc     = \'Required Value\'\ncodebuild_service_role_arn = \'Required Value\'\ncodebuild_source_type      = \'Required Value\'\ncodebuild_source_url       = \'Required Value\'\ncodebuild_vpc              = \'Required Value\'\ncodebuild_subnet_list      = [\n    \'Required_Value_1\'\n    \'Required_Value_2\'\n]\ncodebuild_sg_list          = [\n    \'Required_Value_1\'\n    \'Required_Value_2\'\n]\n\nTerraform Project Optional Variables:\n=====================================\ncodebuild_timeout             = 15\ncodebuild_badge               = \'True\'\ncodebuild_artifact_type       = \'NO_ARTIFACTS\'\ncodebuild_artifact_encryption = \'False\'\ncodebuild_encryption_key      = \'NULL\'\ncodebuild_artifact_bucket     = \'NULL\'\ncodebuild_artifact_path       = \'/\'\ncodebuild_source_branch       = \'master\'\ncodebuild_buildspec           = \'buildspec.yml\'\ncodebuild_buildspec_type      = \'FILENAME\'\ncodebuild_env_compute_type    = \'BUILD_GENERAL1_SMALL\'\ncodebuild_env_image           = \'aws/codebuild/standard:1.0\'\ncodebuild_env_type            = \'LINUX_CONTAINER\'\n```\n\n<br><br>\n\n### `magicdoc tf show outputs`\n\nThe `magicdoc tf show outputs` command will display an output of all terraform outputs that were found in the `outputs.tf` file within the target project directory. Magicdoc will use the outputs that it was able to parse from any found outputs.tf files when constructing the readme documentation.\n\n<br>\n\n```yaml\nmagicdoc tf show outputs:\n  Examples:\n    magicdoc tf show outputs\n    magicdoc tf -d /path/to/module/sourcecode show outputs\n  Arguments: None\n  Options:\n      include_examples:\n          Description: Instructs the variables execution environment to include and parse files in any `example` or `examples` subdirectories located in the parent target project directory. By default directories named `example`, or `examples` are excluded from the file/variable/output search results, and are not parsed or included.\n          Value: bool\n          Flag: --include_examples, -i\n          Environment Variable: MAGICDOC_TF_SHOW_VARIABLES_INCLUDE_EXAMPLES\n          Required: No\n          Default: False\n```\n\n<br>\n\n```bash\n  Environment: Verbose Attribute Set:                False\n  Log: Verbose Attribute Set:                        False\n  Environment: Verbose Level Attribute Set:          debug\n\nMagicDoc Terraform Project Outputs Summary:\n===========================================\n  Include Example Directories:                       False\n\nMagicDoc [tf show outputs] Command Environment:\nGathering Terraform Project Variables...\n\nParsing outputs.tf files for Terraform outputs...\nScanning project directory for defined module outputs...\nTerraform output search target directory location: /Volumes/MacData/Work/CloudMage/Terraform/TF-AWS-CodeBuild-Module\n -> 3 terraform project outputs found in target project: /Volumes/MacData/Work/CloudMage/Terraform/TF-AWS-CodeBuild-Module\n\ncodebuild_project_id        = {\n    aws_codebuild_project.this.id\n}\n\ncodebuild_project_arn       = {\n    aws_codebuild_project.this.arn\n}\n\ncodebuild_project_badge_url = {\n    aws_codebuild_project.this.badge_url\n}\n```\n\n<br><br>\n\n### `magicdoc tf show graph`\n\nThe `magicdoc tf show graph` command will tell magicdoc to initialize the terraform project directory by issuing a `terraform init` on the directory. It will then run a `terraform graph` command and store the dot formatted graph data, and display it to the screen. Magicdoc will use this data to automatically construct, and render the terraform graph in PNG format if the `dot` binary file can be found within the executing systems path. If magicdoc is successful in rendering the dot graph into a PNG file, the file will be placed into a images directory in the target project path, and then included in the readme documentation when renderend.\n\n<br>\n\n```yaml\nmagicdoc tf show graph:\n  Examples:\n    magicdoc tf show graph\n    magicdoc tf -d /path/to/module/sourcecode show graph\n  Arguments: None\n  Options:\n      overwrite:\n          Description: Instructs magicdoc to overwrite any existing .terraform directory if one is already found in the target project directory. If magicdoc does not find a .terraform directory, it will perform the `terraform init` run the graph dot file generation, and then remove the .terraform directory that it created in generating the graph data.\n          Value: bool\n          Flag: --overwrite, -o\n          Environment Variable: MAGICDOC_TF_SHOW_GRAPH_OVERWRITE\n          Required: No\n          Default: False\n```\n\n<br>\n\n```bash\nMagicDoc Terraform Project Graph dot Structure:\n===============================================\n  Overwrite Existing .terraform Directory:           False\n\nMagicDoc [tf show graph] Command Environment:\nGenerating Terraform Project Graph...\n\nGenerating terraform graph dot object...\nCLS->TFMagicDoc.terraform_init: Executing `terraform init` on target project directory: /Volumes/MacData/Work/CloudMage/Terraform/TF-AWS-CodeBuild-Module\n\nInitializing the backend...\n\nInitializing provider plugins...\n- Checking for available provider plugins...\n- Downloading plugin for provider "aws" (hashicorp/aws) 2.49.0...\n\nThe following providers do not have any version constraints in configuration,\nso the latest version was installed.\n\nTo prevent automatic upgrades to new major versions that may contain breaking\nchanges, it is recommended to add version = "..." constraints to the\ncorresponding provider blocks in configuration, with the constraint strings\nsuggested below.\n\n* provider.aws: version = "~> 2.49"\n\nTerraform has been successfully initialized!\n\nYou may now begin working with Terraform. Try running "terraform plan" to see\nany changes that are required for your infrastructure. All Terraform commands\nshould now work.\n\nIf you ever set or change modules or backend configuration for Terraform,\nrerun this command to reinitialize your working directory. If you forget, other\ncommands will detect it and remind you to do so if necessary.\nCLS->TFMagicDoc.terraform_init: Execution of `terraform init` completed successfully against the target directory: /Volumes/MacData/Work/CloudMage/Terraform/TF-AWS-CodeBuild-Module\ndigraph {\n        compound = "true"\n        newrank = "true"\n        subgraph "root" {\n                "[root] aws_codebuild_project.this" [label = "aws_codebuild_project.this", shape = "box"]\n                "[root] aws_codebuild_webhook.this" [label = "aws_codebuild_webhook.this", shape = "box"]\n                "[root] local.buildspec_file" [label = "local.buildspec_file", shape = "note"]\n                "[root] local.codebuild_compute_env" [label = "local.codebuild_compute_env", shape = "note"]\n                "[root] local.prebuilt_buildspec" [label = "local.prebuilt_buildspec", shape = "note"]\n                "[root] output.codebuild_project_arn" [label = "output.codebuild_project_arn", shape = "note"]\n                "[root] output.codebuild_project_badge_url" [label = "output.codebuild_project_badge_url", shape = "note"]\n                "[root] output.codebuild_project_id" [label = "output.codebuild_project_id", shape = "note"]\n                ...etc\n        }\n}\n```\n\n<br><br>\n\n### `magicdoc tf show git`\n\nThe `magicdoc tf show git` command will display the git repository information that it was able to parse from the existing `.git/config` directory if one is present in the target project directory. Magicdoc will use the information that its able to parse from the .git/config file to form the requests to github to query for latest release data prior to document generation.\n\n<br>\n\n```yaml\nmagicdoc tf show git:\n  Examples:\n    magicdoc tf show git\n    magicdoc tf -d /path/to/module/sourcecode show git\n  Arguments: None\n  Options:\n      auth:\n          Description: Passed a git repository personal access token to magicdoc, and is used to construct the appropriate header auth request when requesting data from the configured git repository.\n          Value: token\n          Flag: --auth, -a\n          Environment Variable: MAGICDOC_TF_SHOW_REPO_AUTH\n          Required: No\n          Default: None\n```\n\n<br>\n\n```bash\nMagicDoc Terraform Project Git Config:\n======================================\n\nMagicDoc [tf show git] Command Environment:\nParsing Terraform Project Git Config...\n\nAttempting to parse target project git config...\n{\n    url       = \'git@github.com:CloudMage-TF/TF-AWS-Test-Module.git\'\n    namespace = \'CloudMage-TF\'\n    name      = \'TF-AWS-Test-Module.git\'\n    provider  = \'github.com\'\n}\n```\n\n<br><br>\n\n### `magicdoc tf show repo`\n\nThe `magicdoc tf show repo` command will display the git repository information that it was able to get from performing an API request to the repository found in the .git/config directory. The API collects just relevant information that was determined to be useful in aiding to provide data to the readme document that will be constructed. If the repository is a private repository an auth token can be passed using the -a flag.\n\n<br>\n\n```yaml\nmagicdoc tf show repo:\n  Examples:\n    magicdoc tf show repo\n    magicdoc tf show repo -a "0123456789109876543210"\n    magicdoc tf -d /path/to/module/sourcecode show repo\n  Arguments: None\n  Options:\n      auth:\n          Description: Passed a git repository personal access token to magicdoc, and is used to construct the appropriate header auth request when requesting data from the configured git repository.\n          Value: token\n          Flag: --auth, -a\n          Environment Variable: MAGICDOC_TF_SHOW_REPO_AUTH\n          Required: No\n          Default: None\n```\n\n<br>\n\n```bash\nMagicDoc Terraform Project Latest Release:\n==========================================\n  Git Authentication Token Provided:                 True\n\n\nMagicDoc [tf show release] Command Environment:\nSending Terraform Project Git Release Request...\n\n\nProject Latest Release:\n=======================\n  TF-AWS-ResourceNaming-Module Latest Release:            v1.0.5\n```\n\n<br><br>\n\n### `magicdoc tf show tree`\n\nThe `magicdoc tf show tree` command will construct and display an ascii style directory tree listing for the target project directory. During the data gathering stage, magicdoc will construct this tree view of the target project directory and include the output into the readme documentation.\n\n<br>\n\n```yaml\nmagicdoc tf show tree:\n  Examples:\n    magicdoc tf show tree\n    magicdoc tf -d /path/to/module/sourcecode show tree\n  Arguments: None\n  Options: None\n```\n\n<br>\n\n```bash\n  Environment: Verbose Attribute Set:                False\n  Log: Verbose Attribute Set:                        False\n  Environment: Verbose Level Attribute Set:          debug\n  Log: Verbose Level Attribute Set:                  debug\n  Environment: Template Path Set:                    /Volumes/MacData/Work/CloudMage/PythonLibs/PyPkgs-MagicDoc/magicdoc/templates\n  Environment: Directory Recursion Set:              False\n\nScanning project directory for terraform .tf and .tfvar files...\nUsage: main.py tf create [OPTIONS] COMMAND [ARGS]...\n\n  Create a Magicdoc config or Documentation for your project.\n\n\nMagicDoc Terraform Project Directory Tree Structure:\n====================================================\n\nMagicDoc [tf show tree] Command Environment:\nGathering Terraform Project Outputs...\n\n.\n├── outputs.tf\n├── .DS_Store\n├── Makefile\n├── README.md\n├── .gitignore\n├── variables.tf\n└── magicdoc.yaml\n```\n\n<br><br>\n\n## MagicDoc TF Create Commands\n\nThe `magicdoc tf create` command is a container for create subcommands. By itself this command will not return anything other then the help menu.\n\n<br>\n\n```bash\nUsage: magicdoc.py create [OPTIONS] COMMAND [ARGS]...\n\n  Environment: Verbose Attribute Set:                False\n  Log: Verbose Attribute Set:                        False\n  Environment: Verbose Level Attribute Set:          debug\n  Log: Verbose Level Attribute Set:                  debug\n  Environment: Template Path Set:                    /Volumes/MacData/Work/CloudMage/PythonLibs/PyPkgs-MagicDoc/magicdoc/templates\n  Environment: Directory Recursion Set:              False\n\nScanning project directory for terraform .tf and .tfvar files...\nUsage: main.py tf create [OPTIONS] COMMAND [ARGS]...\n\n  Create a Magicdoc config or Documentation for your project.\n\nOptions:\n  --help  Show this message and exit.\n\nCommands:\n  config  Create a magicdoc project configuration file.\n  doc     Create a terraform module or project README.md file.\n```\n\n<br>\n\n### `magicdoc tf create config`\n\nCommand to generate a magicdoc config file for the target terraform project. The config file allows a user to supply additional information that will be rendered into the generated readme document. The command takes 1 option, being the type, identifying the type of project/the template used to render the document. The valid available option choices are `module` and `root`, although currently only module will work as a root template is still currently in development. If no option is selected, then magicdoc will default to module. The `create config` command also takes a single optional argument that will allow the command issuer the ability to name the config file something other then magicdoc.yaml. If a custom config name is specified then subsequent commands to generate or regenerate the document will require that the `magicdoc tf -c /path/to/confg` option is set. The file must be a yaml file extention, and this is forced, meaning that even if a custom file name is provided without a yaml extention, magicdoc will remove the extention and append .yaml to the custom config name.\n\n<br>\n\n```yaml\nmagicdoc tf create config -t <type> <filename>:\n  Examples:\n    magicdoc tf create config\n    magicdoc tf create config - module\n    magicdoc tf create config -t module project.yaml\n    magicdoc tf -d /path/to/module/sourcecode create config\n    magicdoc tf -d /path/to/module/sourcecode create config -t module\n    magicdoc tf -d /path/to/module/sourcecode create config -t module config.yaml\n  Arguments:\n      filename:\n          Description: The custom name that will be given to the config file that is produced and written to the target project directory.\n          Value: Any string value that will be set as the project config file name.\n          Required: No, If filename is specified, then magicdoc will use the provided filename to name the output config file.\n          Default: magicdoc.yaml\n  Options:\n      type:\n          Description: The type of project that magicdoc will parse and build documentation for. When using the terraform provider, `module` and `root` are the only valid options.\n          Note: Magicdoc currently only supports module documentation generation. A root template is currently in development.\n          Value: module, root\n          Flag: --type, -t\n          Environment Variable: MAGICDOC_TF_CREATE_CONFIG_TYPE\n          Required: No\n          Default: Module\n```\n\n<br>\n\n```bash\nMagicDoc Terraform Documentation Creator:\n=========================================\n  Config Output File:                                /Volumes/MacData/Work/Terraform/Modules/Documentation/TF-TestModule/magicdoc.yaml\n\nMagicDoc Terraform Module Config Generator: [tf create config -t module <output_file>]:\nAttempting to Gather Terraform Project Data...\n\nScanning project directory for defined module variables...\n\nGenerating Terraform Module Magicdoc Config: magicdoc.yaml\n\nMagicdoc Terraform Module config file: magicdoc.yaml has been successfully created in :/Volumes/MacData/Work/Terraform/Modules/Documentation/TF-TestModule!\n```\n\n<br><br>\n\n### `magicdoc tf create doc`\n\nThe `magicdoc tf create doc` command is the magic behind magicdoc. When this command is issued, magicdoc will run a data collection on all of the project outputs, variables, files, graph, git repository and release data, build the tree, and create the readme document. If a `README.md` file already exists in the target project directory then magicdoc will automatically backup this file, timestamp it, rename it as README_<date>_<timestamp>.bak and generate the new README file as `README.md` in the target project directory.\n\n<br>\n\n```yaml\nmagicdoc tf create doc <-t type>:\n  Examples:\n    magicdoc tf create doc\n    magicdoc tf create doc --help\n    magicdoc tf create doc -t module\n    magicdoc tf -d /path/to/module/sourcecode create doc -t module\n    magicdoc tf -d /path/to/module/sourcecode create doc -t module -a "0123456789109876543210"\n  Arguments: None\n  Options:\n      type:\n          Description: The type of project that magicdoc will parse and build documentation for. When using the terraform provider, `module` and `root` are the only valid options.\n          Note: Magicdoc currently only supports module documentation generation. A root template is currently in development.\n          Value: module, root\n          Flag: --type, -t\n          Environment Variable: MAGICDOC_TF_CREATE_CONFIG_TYPE\n          Required: No\n          Default: Module\n      auth:\n          Description: Passed a git repository personal access token to magicdoc, and is used to construct the appropriate header auth request when requesting data from the configured git repository.\n          Value: token\n          Flag: --auth, -a\n          Environment Variable: MAGICDOC_TF_SHOW_REPO_AUTH\n          Required: No\n          Default: None\n```\n\n<br>\n\n```bash\nMagicDoc Terraform Documentation Creator:\n=========================================\n  Output File:                                       README.md\n\nMagicDoc Terraform Module Doc Generator: [tf create doc -t module]:\nAttempting to Gather Terraform Project Data...\n\nMagicDoc git config file not found in /Volumes/MacData/Work/CloudMage/Terraform/TF-AWS-CodeBuild-Module\nNo git information about the target project can be provided at this time.\nScanning project directory for defined module variables...\nScanning project directory for defined module outputs...\n\nMagicDoc Terraform Documentation Creator:\n=========================================\n  Output File:                                       README.md\n\nMagicDoc Terraform Module Doc Generator: [tf create doc -t module]:\nAttempting to Gather Terraform Project Data...\n\nMagicDoc git config file not found in /Volumes/MacData/Work/CloudMage/Terraform/TF-AWS-CodeBuild-Module\nNo git information about the target project can be provided at this time.\nScanning project directory for defined module variables...\nScanning project directory for defined module outputs...\n\nGenerating Terraform Module Readme Documentation...\nMagicdoc Terraform Module documentation file: README.md has been successfully created in :/Volumes/MacData/Work/CloudMage/Terraform/TF-AWS-CodeBuild-Module!\n```\n\n<br><br>\n',
    'author': 'Rich Nason',
    'author_email': 'rnason@cloudmage.io',
    'maintainer': 'Rich Nason',
    'maintainer_email': 'rnason@cloudmage.io',
    'url': 'https://github.com/TheCloudMage/PyPkgs-MagicDoc',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)
